#!/usr/bin/python3

# This code is available for use under CC0 (Creative Commons 0 - universal).
# You can copy, modify, distribute and perform the work, even for commercial
# purposes, all without asking permission. For more information, see LICENSE.md or
# https://creativecommons.org/publicdomain/zero/1.0/

# usage:
# opts = Picker(
#    title = 'Delete all files',
#    options = ["Yes", "No"]
# ).getSelected()

# returns a simple list
# cancel returns False

import curses
import shutil
import signal
from curses import wrapper

class Picker:
    """Allows you to select from a list with curses"""
    stdscr = None
    win = None
    title = ""
    arrow = ""
    footer = ""
    more = ""
    c_selected = ""
    c_empty = ""

    cursor = 0
    offset = 0
    selected = 0
    selcount = 0
    aborted = False

    window_height = shutil.get_terminal_size().lines - 10
    window_width = shutil.get_terminal_size().columns - 20
    all_options = []
    length = 0

    def curses_start(self):
        self.stdscr = curses.initscr()
        curses.noecho()
        curses.cbreak()
        self.win = curses.newwin(
            5 + self.window_height,
            self.window_width,
            2,
            4
        )

    def sigwinch_handler(self, n, frame):
        self.window_height = shutil.get_terminal_size().lines - 10
        self.window_width = shutil.get_terminal_size().columns - 20
        curses.endwin()
        self.stdscr.clear()
        self.stdscr = curses.initscr()
        self.win = curses.newwin(
            5 + self.window_height,
            self.window_width,
            2,
            4
        )

    def curses_stop(self):
        curses.nocbreak()
        self.stdscr.keypad(0)
        curses.echo()
        curses.endwin()

    def getSelected(self):
        if self.aborted == True:
            return( False )

        ret_s = [x for x in self.all_options if x["selected"]]
        ret = [x["label"] for x in ret_s]
        return(ret)

    def redraw(self):
        self.win.clear()
        self.win.border(
            self.border[0], self.border[1],
            self.border[2], self.border[3],
            self.border[4], self.border[5],
            self.border[6], self.border[7]
        )
        self.win.addstr(
            self.window_height + 4, 5, " " + self.footer + " "
        )

        position = 0
        range = self.all_options[self.offset:self.offset+self.window_height+1]
        for option in range:
            if option["selected"] == True:
                line_label = self.c_selected + " "
            else:
                line_label = self.c_empty + " "

            if len(option["label"]) > (self.window_width - 20):
                reduced = option["label"][:self.window_width - 20] + "..."
            else:
                reduced = option["label"]
            self.win.addstr(position + 2, 5, line_label + reduced)
            position = position + 1

        # hint for more content above
        if self.offset > 0:
            self.win.addstr(1, 5, self.more)

        # hint for more content below
        if self.offset + self.window_height <= self.length - 2:
            self.win.addstr(self.window_height + 3, 5, self.more)

        self.win.addstr(0, 5, " " + self.title + " ")
        self.win.addstr(
            0, self.window_width - 8,
            " " + str(self.selcount) + "/" + str(self.length) + " "
        )
        self.win.addstr(self.cursor + 2,1, self.arrow)
        self.win.refresh()

    def check_cursor_up(self):
        if self.cursor < 0:
            self.cursor = 0
            if self.offset > 0:
                self.offset = self.offset - 1

    def check_cursor_down(self):
        if self.cursor >= self.length:
            self.cursor = self.cursor - 1

        if self.cursor > self.window_height:
            self.cursor = self.window_height
            self.offset = self.offset + 1

            if self.offset + self.cursor >= self.length:
                self.offset = self.offset - 1

    def curses_loop(self, stdscr):
        while 1:
            self.redraw()
            c = stdscr.getch()

            if c == ord('q') or c == ord('Q'):
                self.aborted = True
                break
            elif c == curses.KEY_UP:
                self.cursor = self.cursor - 1
            elif c == curses.KEY_DOWN:
                self.cursor = self.cursor + 1
            #elif c == curses.KEY_PPAGE:
            #elif c == curses.KEY_NPAGE:
            elif c == ord(' '):
                self.all_options[self.selected]["selected"] = \
                    not self.all_options[self.selected]["selected"]
            elif c == 10:
                break

            # deal with interaction limits
            self.check_cursor_up()
            self.check_cursor_down()

            # compute selected position only after dealing with limits
            self.selected = self.cursor + self.offset

            temp = self.getSelected()
            self.selcount = len(temp)

    def __init__(self, options, title='Select', arrow="-->",
                 footer="Space = toggle, Enter = accept, q = cancel",
                 more="...", border="||--++++", c_selected="[X]", c_empty="[ ]", checked="[ ]"):
        self.title = title
        self.arrow = arrow
        self.footer = footer
        self.more = more
        self.border = border
        self.c_selected = c_selected
        self.c_empty = c_empty

        self.all_options = []

        for option in options:
            self.all_options.append({
                "label": option,
                "selected": True if (option in checked) else False
            })
            self.length = len(self.all_options)

        self.curses_start()

        signal.signal(signal.SIGWINCH, self.sigwinch_handler)
        curses.wrapper( self.curses_loop )
        self.curses_stop()



import json
import os

DEFAULT_CONFIG = {
    "username": "",
    "base_address": "https://studip.uos.de/plugins.php/restipplugin",
    "local_path": "~/studip",
    "interval": 1200,
    "last_check": -1,
    "courses_selected": False,
    "selected_courses": []
}


CONFIG_FILE = os.path.expanduser(
    os.path.join("~", ".config", "studdp", 'config.json'))

if not os.path.exists(CONFIG_FILE):
    os.makedirs(os.path.dirname(CONFIG_FILE), exist_ok=True)
    with open(CONFIG_FILE, "w") as f:
        json.dump(DEFAULT_CONFIG, f, sort_keys=True, indent=4 * ' ')
with open(CONFIG_FILE, 'r') as rfile:
    CONFIG = json.load(rfile)



#!/usr/bin/python3

"""
StudDP downloads files from Stud.IP.
"""

import json
import logging
import os
import shutil
import signal
import time
import sys
import requests
import re
import optparse
from distutils.util import strtobool
import keyring
import getpass
import atexit
from .picker import Picker
from .APIWrapper import APIWrapper
from . import CONFIG
from . import CONFIG_FILE

LOG = logging.getLogger(__name__)
LOG_PATH = os.path.expanduser(os.path.join('~', '.studdp'))
PID_FILE = os.path.expanduser(os.path.join('~', '.studdp', 'studdp.pid'))
WIN_INVALID_CHARACTERS = [":", "<", ">", "|", "\?", "\*"]


class StudDP(object):
    """
    The main program loops until interrupted.
    Every time files were changed after the last check, they are downloaded.
    Files are also downloaded if they do not exist locally.
    """

    def __init__(self,
                 config,
                 api_helper,
                 daemonize=False,
                 on_windows=False,
                 update=False):
        """
        Initializes the API and the update frequencies.
        """
        self.config = config
        self.interval = self.config['interval']
        self.api = api_helper
        self.daemonize = daemonize
        self.on_windows = on_windows
        self.update = update

    def _needs_download(self, document):
        """
        Checks if a download of the document is needed.
        """
        return ((int(document['chdate']) > self.config['last_check']) and
                self.update) or not \
            os.path.exists(os.path.join(document['path'],
                                        document['filename']))

    def __call__(self):
        """
        Starts the main loop and checks
        periodically for document changes and downloads.
        """
        while True:
            courses = self.api.get_courses()

            if not self.config['courses_selected']:
                LOG.info("Updating course selection")
                titles = map(lambda x: x["title"], courses)
                selection = Picker(
                    title="Select courses to download",
                    options=titles,
                    checked=self.config['selected_courses']).getSelected()
                if not selection:
                    self.config["courses_selected"] = True
                    return
                self.config['selected_courses'] = selection

            LOG.info('Checking courses.')
            for course in courses:
                title = course['title']
                LOG.debug('Course: %s', title)

                if title in self.config['selected_courses']:
                    LOG.info('Checking files for %s', title)
                    documents = self.api.get_documents(course)
                    for document in documents:
                        if self.on_windows:  # Salt Path
                            for char in WIN_INVALID_CHARACTERS:
                                document["path"] = re.sub(
                                    char, "", document["path"])
                                document["filename"] = re.sub(
                                    char, "", document["filename"])
                        if self._needs_download(document):
                            path = os.path.join(
                                document['path'], document['filename'])
                            LOG.info('Downloading %s...', path)
                            os.makedirs(document['path'], exist_ok=True)
                            with open(path, 'wb') as docfile:
                                self.api.download_document(document, docfile)
                            LOG.debug('Saved %s', path)
                else:
                    LOG.debug('Skipping files for %s', title)
            self.config['last_check'] = time.time()
            self.config['courses_selected'] = True
            LOG.info('Done checking.')
            if not self.daemonize:
                return
            time.sleep(self.interval)


def _setup_logging(log_to_stdout=False):
    """
    Sets up the logging handlers.
    """
    os.makedirs(LOG_PATH, exist_ok=True)
    file_handler_info = logging.FileHandler(os.path.join(LOG_PATH, 'info.log'))
    file_handler_info.setLevel(logging.DEBUG)
    file_handler_info.setFormatter(logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
    LOG.addHandler(file_handler_info)
    if log_to_stdout:
        out = logging.StreamHandler(sys.stdout)
        out.setLevel(logging.INFO)
        out.setFormatter(logging.Formatter(
            '%(name)s - %(levelname)s - %(message)s'))
        LOG.addHandler(out)
    err = logging.StreamHandler(sys.stderr)
    err.setLevel(logging.ERROR)
    err.setFormatter(logging.Formatter(
        '%(name)s - %(levelname)s - %(message)s'))
    LOG.addHandler(err)
    LOG.setLevel(logging.DEBUG)
    LOG.info('Logging initialized.')


def _get_password(username, force_update=False):
    if username == "":
        print("No username provided. "
              "Please configure ~/.config/studdp/config.json first")
        exit()
    LOG.info("Querying for password")
    password = keyring.get_password("StudDP", username)
    if not password or force_update:
        password = getpass.getpass(
            "Please enter password for user %s: " % username)
        LOG.info("Adding new password to keyring")
        keyring.set_password("StudDP", username, password)
    return password


def _exit_func():
    """
    Ensures clean exit by writing the current configuration file and
    deleting the pid file.
    """
    LOG.info('Invoking exit.')
    with open(CONFIG_FILE, 'w') as wfile:
        LOG.info('Writing config.')
        json.dump(CONFIG, wfile, sort_keys=True, indent=4 * ' ')
    os.unlink(PID_FILE)
    LOG.info('Exiting.')


def _parse_args():
    parser = optparse.OptionParser()
    parser.add_option("-c", "--config",
                      action="store_true", dest="regenerate", default=False,
                      help="change course selection")
    parser.add_option("-v", "--verbose",
                      action="store_true", dest="log_to_stdout", default=False,
                      help="print log to stdout")
    parser.add_option("-d", "--daemonize",
                      action="store_true", dest="daemonize", default=False,
                      help="start as daemon. Use stopDP to end thread.")
    parser.add_option("-w", "--windows",
                      action="store_true", dest="on_windows", default=False,
                      help="remove characters that are forbidden in windows paths")
    parser.add_option("-u", "--update",
                      action="store_true", dest="update_courses", default=False,
                      help="update files when they are updated on StudIP")
    parser.add_option("-p", "--password",
                      action="store_true", dest="update_password", default=False,
                      help="force password update")
    return parser.parse_args()


def main():
    (options, args) = _parse_args()
    _setup_logging(options.log_to_stdout)

    username = CONFIG["username"]
    password = _get_password(username, options.update_password)

    os.makedirs(os.path.dirname(PID_FILE), exist_ok=True)
    with open(PID_FILE, 'w') as pid_file:
        pid_file.write(str(os.getpid()))

    atexit.register(_exit_func)

    if options.regenerate:
        CONFIG["courses_selected"] = False


    api_helper = APIWrapper((username, password), CONFIG["base_address"],
                            CONFIG["local_path"])

    StudDP(CONFIG, api_helper, options.daemonize, options.on_windows,
           options.update_courses)()

if __name__ == "__main__":
    main()



import json
import logging
import os
import shutil
import signal
import time
import sys
import requests
import time

def retryUntilCondition(condition):
  def decorate(function):
    def f(*args, **kwargs):
      timeout = time.time() + 30
      while True:
        result = function(*args, **kwargs)
        if condition(result):
          return result
        elif time.time() > timeout:
          raise Exception("Unable to connect.")
          return
        time.sleep(0.5)
    return f
  return decorate

def responseIs200(response):
  return response.status_code == 200

class APIWrapper(object):
    """
    An API wrapper for the Stud.IP Rest.API.
    See studip.github.io/studip-rest.ip/ for details.
    """

    def __init__(self, auth, base_address, local_path):

        self._auth = auth
        self._base_address = base_address
        self.local_path = os.path.expanduser(local_path)

    def url(self, route):
        """
        Creates an URL from the configuration and the route.
        """
        return "%s%s" % (self._base_address, route)


    @retryUntilCondition(responseIs200)
    def get(self, route, stream=False):
        """
        Performs a GET request with the authentication from the configuration.
        Will raise errors that have to be handled by the user.
        """
        return requests.get(self.url(route), auth=self._auth, stream=stream)


    def get_courses(self):
        """
        Gets a list of courses.
        """
        return json.loads(self.get('/api/courses').text)["courses"]


    def get_course_folders(self, course):
        """
        Gets a list of document folders for a given course.
        """
        return json.loads(self.get('/api/documents/%s/folder' \
                                    % course['course_id']).text)['folders']


    def get_documents(self, course):
        """
        Gets a list of documents and folders inside a folder.
        """
        documents = []
        folders = self.get_course_folders(course)
        for i, folder in enumerate(folders):
            folders[i]['path'] = os.path.join(self.local_path, course['title'])

        while folders:
            folder = folders.pop()
            try:
                path = '/api/documents/%s/folder/%s' \
                        % (course['course_id'], folder['folder_id'])
                temp = json.loads(self.get(path).text)
            except (ValueError, AttributeError):
                raise Exception('Error on loading %s.' % path)
                continue

            for key in ['folders', 'documents']:
                for i in range(len(temp[key])):
                    temp[key][i]['path'] = os.path.join(folder['path'],
                                                        folder['name'])
            documents += temp['documents']
            folders += temp['folders']
        return documents


    def download_document(self, document, docfile):
        """
        Downloads the document to docfile.
        """
        shutil.copyfileobj(self.get('/api/documents/%s/download' % document['document_id'], stream=True).raw, docfile)



# -*- coding: utf-8 -*-
from setuptools import setup
from os.path import expanduser, join
home = expanduser("~")

setup(
    name='StudDP',
    version='1.1.7',
    author='Maxim Schuwalow',
    author_email='mschuwalow@uos.de',
    packages=['studdp'],
    install_requires=[
        "cffi >= 1.9.1",
        "cryptography >= 1.6",
        "idna >= 2.1",
        "keyring >= 10.1",
        "pyasn1 >= 0.1.9",
        "pycparser >= 2.17",
        "requests >= 2.12.3",
        "SecretStorage >= 2.3.1",
        "six >= 1.10.0"
    ],
    url='https://github.com/MSchuwalow/StudDP.git',
    license='MIT',
    description='StudIP file downloader in python',
    long_description=open('README.md').read(),
    keywords="StudIP Downloader Osnabrueck UOS utility",
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "StudDP = studdp.StudDP:main",
            "stopDP = studdp.stopDP:stop",
        ],
    },
)
